"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Base = void 0;
const BaseCrud_1 = require("./BaseCrud");
const errors_1 = require("./errors");
class Base extends BaseCrud_1.BaseCrud {
    get baseProps() {
        return {};
    }
    get props() {
        return {};
    }
    get combinedProps() {
        return Object.assign(Object.assign({}, this.baseProps), this.props);
    }
    constructor(parentIdsOrThisId, id) {
        super();
        if (this.combinedProps.db === undefined)
            throw Error('db does not assigned');
        if (this.combinedProps.collectionName === undefined)
            throw Error('collectionName has not set');
        const parentDocumentRef = (() => {
            /**
             * parentIdsOrThisId: 3 patterns (undefined, string, [string, ...string[]])
             *                         x
             *       id         : 2 patterns (undefined, string)
             *                         ||
             *                    6 patterns
             */
            if (parentIdsOrThisId === undefined) {
                if (id === undefined) {
                    // e.g. new User()
                    return this.combinedProps.db;
                }
                else if (typeof id === 'string') {
                    // e.g. new User(undefined, userId)
                    throw errors_1.ERRORS.INVALID_ARGS();
                }
            }
            else if (typeof parentIdsOrThisId === 'string') {
                if (id === undefined) {
                    // e.g. new User(userId)
                    return this.combinedProps.db;
                }
                else if (typeof id === 'string') {
                    // e.g. new Post(userId, postId)
                    throw errors_1.ERRORS.INVALID_ARGS();
                }
            }
            else if (Array.isArray(parentIdsOrThisId)) {
                if (this.combinedProps.parent === undefined)
                    throw errors_1.ERRORS.NO_PARENT();
                // e.g. 1 new Post([userId]) or new Post([userId], postId)
                // or
                // e.g. 2. new Comment([userId, postId]) or new Comment([userId, postId], commentId)
                if (parentIdsOrThisId.length === 1) {
                    // e.g. 1 new Post([userId]) or new Post([userId], postId)
                    const [parentId] = parentIdsOrThisId;
                    return new this.combinedProps.parent(parentId).documentReference;
                }
                else {
                    // e.g. 2 new Comment([userId, postId]) or new Comment([userId, postId], commentId)
                    const grandParentIds = parentIdsOrThisId.slice(0, -1);
                    const [parentId] = parentIdsOrThisId.slice(-1);
                    return new this.combinedProps.parent(grandParentIds, parentId).documentReference;
                }
            }
        })();
        this.collectionReference = parentDocumentRef === null || parentDocumentRef === void 0 ? void 0 : parentDocumentRef.collection(this.combinedProps.collectionName);
        this.documentReference =
            id !== undefined
                ? this.collectionReference.doc(id)
                : typeof parentIdsOrThisId === 'string'
                    ? this.collectionReference.doc(parentIdsOrThisId)
                    : undefined;
    }
}
exports.Base = Base;
