"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.BaseCrud = void 0;
const errors_1 = require("./errors");
class BaseCrud {
    mergeIdToSnap(snap) {
        const data = snap.data();
        if (!data)
            return undefined;
        return Object.assign(Object.assign({}, data), { id: snap.id });
    }
    /**
     * @return {Promise<string>} id
     */
    create(data) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.collectionReference == null)
                throw errors_1.ERRORS.NO_COLLECTION_REFERENCE;
            const { id } = data, addData = __rest(data, ["id"]);
            if (id) {
                yield this.collectionReference.doc(id).set(addData);
                return id;
            }
            const addedDocumentReference = yield this.collectionReference.add(addData);
            return addedDocumentReference.id;
        });
    }
    find() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.documentReference == null)
                throw errors_1.ERRORS.NO_DOCUMENT_REFERENCE('Did you mean? getAll');
            return this.documentReference.get().then((snap) => this.mergeIdToSnap(snap));
        });
    }
    all(arg) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.collectionReference == null)
                throw Error();
            const snapshot = yield (arg ? this.collectionReference.limit(arg.limit) : this.collectionReference).get();
            return snapshot.docs.flatMap((doc) => { var _a; return (_a = this.mergeIdToSnap(doc)) !== null && _a !== void 0 ? _a : []; });
        });
    }
    update(data) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.documentReference == null)
                throw errors_1.ERRORS.NO_DOCUMENT_REFERENCE();
            return this.documentReference.update(data);
        });
    }
    destroy() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.documentReference == null)
                throw errors_1.ERRORS.NO_DOCUMENT_REFERENCE();
            return this.documentReference.delete();
        });
    }
}
exports.BaseCrud = BaseCrud;
